# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/error.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import options_pb2 as blickfeld_dot_options__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/error.proto',
  package='blickfeld.protocol',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=_b('\n\x15\x62lickfeld/error.proto\x12\x12\x62lickfeld.protocol\x1a\x17\x62lickfeld/options.proto\"\x8e\x1c\n\x05\x45rror\x12\x34\n\x07unknown\x18\x01 \x01(\x0b\x32!.blickfeld.protocol.Error.UnknownH\x00\x12\x43\n\x0fnot_implemented\x18\x02 \x01(\x0b\x32(.blickfeld.protocol.Error.NotImplementedH\x00\x12\x30\n\x05\x65mpty\x18\x03 \x01(\x0b\x32\x1f.blickfeld.protocol.Error.EmptyH\x00\x12O\n\x15server_implementation\x18\x04 \x01(\x0b\x32..blickfeld.protocol.Error.ServerImplementationH\x00\x12\x43\n\x0finvalid_request\x18\x05 \x01(\x0b\x32(.blickfeld.protocol.Error.InvalidRequestH\x00\x12G\n\x11\x63onnection_closed\x18\x06 \x01(\x0b\x32*.blickfeld.protocol.Error.ConnectionClosedH\x00\x12T\n\x18outdated_server_protocol\x18\x0b \x01(\x0b\x32\x30.blickfeld.protocol.Error.OutdatedServerProtocolH\x00\x12T\n\x18outdated_client_protocol\x18\x0c \x01(\x0b\x32\x30.blickfeld.protocol.Error.OutdatedClientProtocolH\x00\x12=\n\x0cscanner_busy\x18\r \x01(\x0b\x32%.blickfeld.protocol.Error.ScannerBusyH\x00\x12L\n\x14wrong_operation_mode\x18\x0e \x01(\x0b\x32,.blickfeld.protocol.Error.WrongOperationModeH\x00\x12;\n\x0bnot_allowed\x18\x0f \x01(\x0b\x32$.blickfeld.protocol.Error.NotAllowedH\x00\x12\x41\n\x0ehardware_error\x18\x10 \x01(\x0b\x32\'.blickfeld.protocol.Error.HardwareErrorH\x00\x12;\n\x0bsystem_stop\x18\x11 \x01(\x0b\x32$.blickfeld.protocol.Error.SystemStopH\x00\x12\x37\n\tnot_found\x18\x12 \x01(\x0b\x32\".blickfeld.protocol.Error.NotFoundH\x00\x12H\n\x12unknown_error_code\x18\x15 \x01(\x0b\x32*.blickfeld.protocol.Error.UnknownErrorCodeH\x00\x12<\n\x0cnot_in_range\x18\x16 \x01(\x0b\x32$.blickfeld.protocol.Error.NotInRangeH\x00\x12\x44\n\x10time_sync_failed\x18\x17 \x01(\x0b\x32(.blickfeld.protocol.Error.TimeSyncFailedH\x00\x12L\n\x14no_device_discovered\x18\x18 \x01(\x0b\x32,.blickfeld.protocol.Error.NoDeviceDiscoveredH\x00\x1al\n\x07Unknown\x12\x13\n\x0b\x64\x65scription\x18\x01 \x01(\t:L\x82\xa6\x1dHThe server threw an unknown generic error. Description: {description:%s}\x1a\x80\x01\n\x0eNotImplemented:n\x82\xa6\x1djThe request is not implemented on the server side. Check the versions of the client and firmware software.\x1a.\n\x05\x45mpty:%\x82\xa6\x1d!The server sent an empty message.\x1a[\n\x14ServerImplementation:C\x82\xa6\x1d?The implementation on the server failed to process the request.\x1as\n\x0eInvalidRequest\x12\x18\n\x10validation_error\x18\x01 \x01(\t:G\x82\xa6\x1d\x43The sent request did not pass the validation: {validation_error:%s}\x1a@\n\x10\x43onnectionClosed:,\x82\xa6\x1d(The connection was closed by the server.\x1a\x91\x01\n\x16OutdatedServerProtocol\x12\x18\n\x10required_version\x18\x01 \x01(\r:]\x82\xa6\x1dYThe protocol of the client is too new. Please update the device to {required_version:%u}.\x1a\x9a\x01\n\x16OutdatedClientProtocol\x12\x18\n\x10required_version\x18\x01 \x01(\r:f\x82\xa6\x1d\x62The protocol of the server is too new. Please update the client software to {required_version:%u}.\x1a`\n\x0bScannerBusy:Q\x82\xa6\x1dMThe device is currently busy with another operation. Cancel it and try again.\x1a\x84\x01\n\x12WrongOperationMode:n\x82\xa6\x1djThe device is set to a different operation mode.Perhaps it is currently being used by another application.\x1a\x39\n\nNotAllowed:+\x82\xa6\x1d\'The requested operation is not allowed.\x1a\xab\x01\n\rHardwareError:\x99\x01\x82\xa6\x1d\x94\x01The device is in an error state and cannot be operated. Attempt to power cycle the device or use diagnostic software to read-out the hardware state.\x1a\xaf\x01\n\nSystemStop:\xa0\x01\x82\xa6\x1d\x9b\x01The device stopped unexpectedly and can no longer be operated. Attempt to power cycle the device or use diagnostic software to read-out the hardware state.\x1aY\n\x08NotFound:M\x82\xa6\x1dIThe requested data is not available. Please check the request parameters.\x1a\x92\x01\n\x0eTimeSyncFailed\x12\x16\n\x0entp_daemon_log\x18\x01 \x01(\t:h\x82\xa6\x1d\x64The time synchronization failed. The NTP daemon failed or timed out. Log output:\n{ntp_daemon_log:%s}\x1a\xc7\x01\n\x12NoDeviceDiscovered:\xb0\x01\x82\xa6\x1d\xab\x01No device has been discovered. Check the network connection and power supply of the device. The \'discover\' requests may have been blocked by a local or a network firewall.\x1aO\n\x10UnknownErrorCode\x12\x12\n\nerror_code\x18\x01 \x01(\r:\'\x82\xa6\x1d#Unknown Error Code: {error_code:%u}\x1a\x9d\x02\n\nNotInRange\x12\x11\n\tparameter\x18\x01 \x01(\t\x12\x0f\n\x07minimum\x18\x02 \x01(\x02\x12\x0f\n\x07maximum\x18\x03 \x01(\x02\x12\x11\n\trequested\x18\x04 \x01(\x02\x12\x0e\n\x04unit\x18\x05 \x01(\t:\x00:\xb6\x01\x82\xa6\x1d\xb1\x01The requested {parameter:%s} ({requested:%.2f}{unit:%s}) is not within the valid range. The {parameter:%s} has to be in the interval of {minimum:%.2f} < {maximum:%.2f}{unit:%s}.B\x07\n\x05\x65rrorJ\x04\x08\x07\x10\x0bJ\x04\x08\x13\x10\x14J\x04\x08\x14\x10\x15')
  ,
  dependencies=[blickfeld_dot_options__pb2.DESCRIPTOR,])




_ERROR_UNKNOWN = _descriptor.Descriptor(
  name='Unknown',
  full_name='blickfeld.protocol.Error.Unknown',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='description', full_name='blickfeld.protocol.Error.Unknown.description', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035HThe server threw an unknown generic error. Description: {description:%s}'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1319,
  serialized_end=1427,
)

_ERROR_NOTIMPLEMENTED = _descriptor.Descriptor(
  name='NotImplemented',
  full_name='blickfeld.protocol.Error.NotImplemented',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035jThe request is not implemented on the server side. Check the versions of the client and firmware software.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1430,
  serialized_end=1558,
)

_ERROR_EMPTY = _descriptor.Descriptor(
  name='Empty',
  full_name='blickfeld.protocol.Error.Empty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035!The server sent an empty message.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1560,
  serialized_end=1606,
)

_ERROR_SERVERIMPLEMENTATION = _descriptor.Descriptor(
  name='ServerImplementation',
  full_name='blickfeld.protocol.Error.ServerImplementation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035?The implementation on the server failed to process the request.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1608,
  serialized_end=1699,
)

_ERROR_INVALIDREQUEST = _descriptor.Descriptor(
  name='InvalidRequest',
  full_name='blickfeld.protocol.Error.InvalidRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='validation_error', full_name='blickfeld.protocol.Error.InvalidRequest.validation_error', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035CThe sent request did not pass the validation: {validation_error:%s}'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1701,
  serialized_end=1816,
)

_ERROR_CONNECTIONCLOSED = _descriptor.Descriptor(
  name='ConnectionClosed',
  full_name='blickfeld.protocol.Error.ConnectionClosed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035(The connection was closed by the server.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1818,
  serialized_end=1882,
)

_ERROR_OUTDATEDSERVERPROTOCOL = _descriptor.Descriptor(
  name='OutdatedServerProtocol',
  full_name='blickfeld.protocol.Error.OutdatedServerProtocol',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='required_version', full_name='blickfeld.protocol.Error.OutdatedServerProtocol.required_version', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035YThe protocol of the client is too new. Please update the device to {required_version:%u}.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1885,
  serialized_end=2030,
)

_ERROR_OUTDATEDCLIENTPROTOCOL = _descriptor.Descriptor(
  name='OutdatedClientProtocol',
  full_name='blickfeld.protocol.Error.OutdatedClientProtocol',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='required_version', full_name='blickfeld.protocol.Error.OutdatedClientProtocol.required_version', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035bThe protocol of the server is too new. Please update the client software to {required_version:%u}.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2033,
  serialized_end=2187,
)

_ERROR_SCANNERBUSY = _descriptor.Descriptor(
  name='ScannerBusy',
  full_name='blickfeld.protocol.Error.ScannerBusy',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035MThe device is currently busy with another operation. Cancel it and try again.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2189,
  serialized_end=2285,
)

_ERROR_WRONGOPERATIONMODE = _descriptor.Descriptor(
  name='WrongOperationMode',
  full_name='blickfeld.protocol.Error.WrongOperationMode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035jThe device is set to a different operation mode.Perhaps it is currently being used by another application.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2288,
  serialized_end=2420,
)

_ERROR_NOTALLOWED = _descriptor.Descriptor(
  name='NotAllowed',
  full_name='blickfeld.protocol.Error.NotAllowed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035\'The requested operation is not allowed.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2422,
  serialized_end=2479,
)

_ERROR_HARDWAREERROR = _descriptor.Descriptor(
  name='HardwareError',
  full_name='blickfeld.protocol.Error.HardwareError',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035\224\001The device is in an error state and cannot be operated. Attempt to power cycle the device or use diagnostic software to read-out the hardware state.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2482,
  serialized_end=2653,
)

_ERROR_SYSTEMSTOP = _descriptor.Descriptor(
  name='SystemStop',
  full_name='blickfeld.protocol.Error.SystemStop',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035\233\001The device stopped unexpectedly and can no longer be operated. Attempt to power cycle the device or use diagnostic software to read-out the hardware state.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2656,
  serialized_end=2831,
)

_ERROR_NOTFOUND = _descriptor.Descriptor(
  name='NotFound',
  full_name='blickfeld.protocol.Error.NotFound',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035IThe requested data is not available. Please check the request parameters.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2833,
  serialized_end=2922,
)

_ERROR_TIMESYNCFAILED = _descriptor.Descriptor(
  name='TimeSyncFailed',
  full_name='blickfeld.protocol.Error.TimeSyncFailed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ntp_daemon_log', full_name='blickfeld.protocol.Error.TimeSyncFailed.ntp_daemon_log', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035dThe time synchronization failed. The NTP daemon failed or timed out. Log output:\n{ntp_daemon_log:%s}'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2925,
  serialized_end=3071,
)

_ERROR_NODEVICEDISCOVERED = _descriptor.Descriptor(
  name='NoDeviceDiscovered',
  full_name='blickfeld.protocol.Error.NoDeviceDiscovered',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035\253\001No device has been discovered. Check the network connection and power supply of the device. The \'discover\' requests may have been blocked by a local or a network firewall.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3074,
  serialized_end=3273,
)

_ERROR_UNKNOWNERRORCODE = _descriptor.Descriptor(
  name='UnknownErrorCode',
  full_name='blickfeld.protocol.Error.UnknownErrorCode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='error_code', full_name='blickfeld.protocol.Error.UnknownErrorCode.error_code', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035#Unknown Error Code: {error_code:%u}'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3275,
  serialized_end=3354,
)

_ERROR_NOTINRANGE = _descriptor.Descriptor(
  name='NotInRange',
  full_name='blickfeld.protocol.Error.NotInRange',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='parameter', full_name='blickfeld.protocol.Error.NotInRange.parameter', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='minimum', full_name='blickfeld.protocol.Error.NotInRange.minimum', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maximum', full_name='blickfeld.protocol.Error.NotInRange.maximum', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='requested', full_name='blickfeld.protocol.Error.NotInRange.requested', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unit', full_name='blickfeld.protocol.Error.NotInRange.unit', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=True, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\202\246\035\261\001The requested {parameter:%s} ({requested:%.2f}{unit:%s}) is not within the valid range. The {parameter:%s} has to be in the interval of {minimum:%.2f} < {maximum:%.2f}{unit:%s}.'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3357,
  serialized_end=3642,
)

_ERROR = _descriptor.Descriptor(
  name='Error',
  full_name='blickfeld.protocol.Error',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='unknown', full_name='blickfeld.protocol.Error.unknown', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='not_implemented', full_name='blickfeld.protocol.Error.not_implemented', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='empty', full_name='blickfeld.protocol.Error.empty', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='server_implementation', full_name='blickfeld.protocol.Error.server_implementation', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='invalid_request', full_name='blickfeld.protocol.Error.invalid_request', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='connection_closed', full_name='blickfeld.protocol.Error.connection_closed', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='outdated_server_protocol', full_name='blickfeld.protocol.Error.outdated_server_protocol', index=6,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='outdated_client_protocol', full_name='blickfeld.protocol.Error.outdated_client_protocol', index=7,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='scanner_busy', full_name='blickfeld.protocol.Error.scanner_busy', index=8,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='wrong_operation_mode', full_name='blickfeld.protocol.Error.wrong_operation_mode', index=9,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='not_allowed', full_name='blickfeld.protocol.Error.not_allowed', index=10,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hardware_error', full_name='blickfeld.protocol.Error.hardware_error', index=11,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='system_stop', full_name='blickfeld.protocol.Error.system_stop', index=12,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='not_found', full_name='blickfeld.protocol.Error.not_found', index=13,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unknown_error_code', full_name='blickfeld.protocol.Error.unknown_error_code', index=14,
      number=21, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='not_in_range', full_name='blickfeld.protocol.Error.not_in_range', index=15,
      number=22, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time_sync_failed', full_name='blickfeld.protocol.Error.time_sync_failed', index=16,
      number=23, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='no_device_discovered', full_name='blickfeld.protocol.Error.no_device_discovered', index=17,
      number=24, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_ERROR_UNKNOWN, _ERROR_NOTIMPLEMENTED, _ERROR_EMPTY, _ERROR_SERVERIMPLEMENTATION, _ERROR_INVALIDREQUEST, _ERROR_CONNECTIONCLOSED, _ERROR_OUTDATEDSERVERPROTOCOL, _ERROR_OUTDATEDCLIENTPROTOCOL, _ERROR_SCANNERBUSY, _ERROR_WRONGOPERATIONMODE, _ERROR_NOTALLOWED, _ERROR_HARDWAREERROR, _ERROR_SYSTEMSTOP, _ERROR_NOTFOUND, _ERROR_TIMESYNCFAILED, _ERROR_NODEVICEDISCOVERED, _ERROR_UNKNOWNERRORCODE, _ERROR_NOTINRANGE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='error', full_name='blickfeld.protocol.Error.error',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=71,
  serialized_end=3669,
)

_ERROR_UNKNOWN.containing_type = _ERROR
_ERROR_NOTIMPLEMENTED.containing_type = _ERROR
_ERROR_EMPTY.containing_type = _ERROR
_ERROR_SERVERIMPLEMENTATION.containing_type = _ERROR
_ERROR_INVALIDREQUEST.containing_type = _ERROR
_ERROR_CONNECTIONCLOSED.containing_type = _ERROR
_ERROR_OUTDATEDSERVERPROTOCOL.containing_type = _ERROR
_ERROR_OUTDATEDCLIENTPROTOCOL.containing_type = _ERROR
_ERROR_SCANNERBUSY.containing_type = _ERROR
_ERROR_WRONGOPERATIONMODE.containing_type = _ERROR
_ERROR_NOTALLOWED.containing_type = _ERROR
_ERROR_HARDWAREERROR.containing_type = _ERROR
_ERROR_SYSTEMSTOP.containing_type = _ERROR
_ERROR_NOTFOUND.containing_type = _ERROR
_ERROR_TIMESYNCFAILED.containing_type = _ERROR
_ERROR_NODEVICEDISCOVERED.containing_type = _ERROR
_ERROR_UNKNOWNERRORCODE.containing_type = _ERROR
_ERROR_NOTINRANGE.containing_type = _ERROR
_ERROR.fields_by_name['unknown'].message_type = _ERROR_UNKNOWN
_ERROR.fields_by_name['not_implemented'].message_type = _ERROR_NOTIMPLEMENTED
_ERROR.fields_by_name['empty'].message_type = _ERROR_EMPTY
_ERROR.fields_by_name['server_implementation'].message_type = _ERROR_SERVERIMPLEMENTATION
_ERROR.fields_by_name['invalid_request'].message_type = _ERROR_INVALIDREQUEST
_ERROR.fields_by_name['connection_closed'].message_type = _ERROR_CONNECTIONCLOSED
_ERROR.fields_by_name['outdated_server_protocol'].message_type = _ERROR_OUTDATEDSERVERPROTOCOL
_ERROR.fields_by_name['outdated_client_protocol'].message_type = _ERROR_OUTDATEDCLIENTPROTOCOL
_ERROR.fields_by_name['scanner_busy'].message_type = _ERROR_SCANNERBUSY
_ERROR.fields_by_name['wrong_operation_mode'].message_type = _ERROR_WRONGOPERATIONMODE
_ERROR.fields_by_name['not_allowed'].message_type = _ERROR_NOTALLOWED
_ERROR.fields_by_name['hardware_error'].message_type = _ERROR_HARDWAREERROR
_ERROR.fields_by_name['system_stop'].message_type = _ERROR_SYSTEMSTOP
_ERROR.fields_by_name['not_found'].message_type = _ERROR_NOTFOUND
_ERROR.fields_by_name['unknown_error_code'].message_type = _ERROR_UNKNOWNERRORCODE
_ERROR.fields_by_name['not_in_range'].message_type = _ERROR_NOTINRANGE
_ERROR.fields_by_name['time_sync_failed'].message_type = _ERROR_TIMESYNCFAILED
_ERROR.fields_by_name['no_device_discovered'].message_type = _ERROR_NODEVICEDISCOVERED
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['unknown'])
_ERROR.fields_by_name['unknown'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['not_implemented'])
_ERROR.fields_by_name['not_implemented'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['empty'])
_ERROR.fields_by_name['empty'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['server_implementation'])
_ERROR.fields_by_name['server_implementation'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['invalid_request'])
_ERROR.fields_by_name['invalid_request'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['connection_closed'])
_ERROR.fields_by_name['connection_closed'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['outdated_server_protocol'])
_ERROR.fields_by_name['outdated_server_protocol'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['outdated_client_protocol'])
_ERROR.fields_by_name['outdated_client_protocol'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['scanner_busy'])
_ERROR.fields_by_name['scanner_busy'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['wrong_operation_mode'])
_ERROR.fields_by_name['wrong_operation_mode'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['not_allowed'])
_ERROR.fields_by_name['not_allowed'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['hardware_error'])
_ERROR.fields_by_name['hardware_error'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['system_stop'])
_ERROR.fields_by_name['system_stop'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['not_found'])
_ERROR.fields_by_name['not_found'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['unknown_error_code'])
_ERROR.fields_by_name['unknown_error_code'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['not_in_range'])
_ERROR.fields_by_name['not_in_range'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['time_sync_failed'])
_ERROR.fields_by_name['time_sync_failed'].containing_oneof = _ERROR.oneofs_by_name['error']
_ERROR.oneofs_by_name['error'].fields.append(
  _ERROR.fields_by_name['no_device_discovered'])
_ERROR.fields_by_name['no_device_discovered'].containing_oneof = _ERROR.oneofs_by_name['error']
DESCRIPTOR.message_types_by_name['Error'] = _ERROR
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Error = _reflection.GeneratedProtocolMessageType('Error', (_message.Message,), dict(

  Unknown = _reflection.GeneratedProtocolMessageType('Unknown', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_UNKNOWN,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.Unknown)
    ))
  ,

  NotImplemented = _reflection.GeneratedProtocolMessageType('NotImplemented', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_NOTIMPLEMENTED,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.NotImplemented)
    ))
  ,

  Empty = _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_EMPTY,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.Empty)
    ))
  ,

  ServerImplementation = _reflection.GeneratedProtocolMessageType('ServerImplementation', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_SERVERIMPLEMENTATION,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.ServerImplementation)
    ))
  ,

  InvalidRequest = _reflection.GeneratedProtocolMessageType('InvalidRequest', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_INVALIDREQUEST,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.InvalidRequest)
    ))
  ,

  ConnectionClosed = _reflection.GeneratedProtocolMessageType('ConnectionClosed', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_CONNECTIONCLOSED,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.ConnectionClosed)
    ))
  ,

  OutdatedServerProtocol = _reflection.GeneratedProtocolMessageType('OutdatedServerProtocol', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_OUTDATEDSERVERPROTOCOL,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.OutdatedServerProtocol)
    ))
  ,

  OutdatedClientProtocol = _reflection.GeneratedProtocolMessageType('OutdatedClientProtocol', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_OUTDATEDCLIENTPROTOCOL,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.OutdatedClientProtocol)
    ))
  ,

  ScannerBusy = _reflection.GeneratedProtocolMessageType('ScannerBusy', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_SCANNERBUSY,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.ScannerBusy)
    ))
  ,

  WrongOperationMode = _reflection.GeneratedProtocolMessageType('WrongOperationMode', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_WRONGOPERATIONMODE,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.WrongOperationMode)
    ))
  ,

  NotAllowed = _reflection.GeneratedProtocolMessageType('NotAllowed', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_NOTALLOWED,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.NotAllowed)
    ))
  ,

  HardwareError = _reflection.GeneratedProtocolMessageType('HardwareError', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_HARDWAREERROR,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.HardwareError)
    ))
  ,

  SystemStop = _reflection.GeneratedProtocolMessageType('SystemStop', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_SYSTEMSTOP,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.SystemStop)
    ))
  ,

  NotFound = _reflection.GeneratedProtocolMessageType('NotFound', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_NOTFOUND,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.NotFound)
    ))
  ,

  TimeSyncFailed = _reflection.GeneratedProtocolMessageType('TimeSyncFailed', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_TIMESYNCFAILED,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.TimeSyncFailed)
    ))
  ,

  NoDeviceDiscovered = _reflection.GeneratedProtocolMessageType('NoDeviceDiscovered', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_NODEVICEDISCOVERED,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.NoDeviceDiscovered)
    ))
  ,

  UnknownErrorCode = _reflection.GeneratedProtocolMessageType('UnknownErrorCode', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_UNKNOWNERRORCODE,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.UnknownErrorCode)
    ))
  ,

  NotInRange = _reflection.GeneratedProtocolMessageType('NotInRange', (_message.Message,), dict(
    DESCRIPTOR = _ERROR_NOTINRANGE,
    __module__ = 'blickfeld.error_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error.NotInRange)
    ))
  ,
  DESCRIPTOR = _ERROR,
  __module__ = 'blickfeld.error_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.Error)
  ))
_sym_db.RegisterMessage(Error)
_sym_db.RegisterMessage(Error.Unknown)
_sym_db.RegisterMessage(Error.NotImplemented)
_sym_db.RegisterMessage(Error.Empty)
_sym_db.RegisterMessage(Error.ServerImplementation)
_sym_db.RegisterMessage(Error.InvalidRequest)
_sym_db.RegisterMessage(Error.ConnectionClosed)
_sym_db.RegisterMessage(Error.OutdatedServerProtocol)
_sym_db.RegisterMessage(Error.OutdatedClientProtocol)
_sym_db.RegisterMessage(Error.ScannerBusy)
_sym_db.RegisterMessage(Error.WrongOperationMode)
_sym_db.RegisterMessage(Error.NotAllowed)
_sym_db.RegisterMessage(Error.HardwareError)
_sym_db.RegisterMessage(Error.SystemStop)
_sym_db.RegisterMessage(Error.NotFound)
_sym_db.RegisterMessage(Error.TimeSyncFailed)
_sym_db.RegisterMessage(Error.NoDeviceDiscovered)
_sym_db.RegisterMessage(Error.UnknownErrorCode)
_sym_db.RegisterMessage(Error.NotInRange)


_ERROR_UNKNOWN._options = None
_ERROR_NOTIMPLEMENTED._options = None
_ERROR_EMPTY._options = None
_ERROR_SERVERIMPLEMENTATION._options = None
_ERROR_INVALIDREQUEST._options = None
_ERROR_CONNECTIONCLOSED._options = None
_ERROR_OUTDATEDSERVERPROTOCOL._options = None
_ERROR_OUTDATEDCLIENTPROTOCOL._options = None
_ERROR_SCANNERBUSY._options = None
_ERROR_WRONGOPERATIONMODE._options = None
_ERROR_NOTALLOWED._options = None
_ERROR_HARDWAREERROR._options = None
_ERROR_SYSTEMSTOP._options = None
_ERROR_NOTFOUND._options = None
_ERROR_TIMESYNCFAILED._options = None
_ERROR_NODEVICEDISCOVERED._options = None
_ERROR_UNKNOWNERRORCODE._options = None
_ERROR_NOTINRANGE._options = None
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/data/point_cloud.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..data import frame_pb2 as blickfeld_dot_data_dot_frame__pb2
from ..data import scanline_pb2 as blickfeld_dot_data_dot_scanline__pb2
from .. import options_pb2 as blickfeld_dot_options__pb2
from ..update import hardware_pb2 as blickfeld_dot_update_dot_hardware__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/data/point_cloud.proto',
  package='blickfeld.protocol.data',
  syntax='proto2',
  serialized_options=_b('\370\001\001'),
  serialized_pb=_b('\n blickfeld/data/point_cloud.proto\x12\x17\x62lickfeld.protocol.data\x1a\x1a\x62lickfeld/data/frame.proto\x1a\x1d\x62lickfeld/data/scanline.proto\x1a\x17\x62lickfeld/options.proto\x1a\x1f\x62lickfeld/update/hardware.proto\"\xb3\x03\n\nPointCloud\x12<\n\x06header\x18\x03 \x01(\x0b\x32*.blickfeld.protocol.data.PointCloud.HeaderH\x00\x12/\n\x05\x66rame\x18\x01 \x01(\x0b\x32\x1e.blickfeld.protocol.data.FrameH\x00\x12\x35\n\x08scanline\x18\x02 \x01(\x0b\x32!.blickfeld.protocol.data.ScanlineH\x00\x1a\xf6\x01\n\x06Header\x12\'\n\x19legacy_cube_serial_number\x18\x01 \x01(\tB\x04\xb0\xb5\x18\x01\x12/\n\x12\x63ube_serial_number\x18\x02 \x01(\tB\x13\xd0\xb5\x18\x01\xaa\xb5\x18\x0b[A-Z2-7]{9}\x12\x15\n\rstart_time_ns\x18\x03 \x01(\x04\x12\x35\n\x10\x66irmware_version\x18\x04 \x01(\tB\x1b\xaa\xb5\x18\x17v[0-9]+\\.[0-9]+\\.[0-9]+\x12\x44\n\x10hardware_variant\x18\x05 \x01(\x0e\x32*.blickfeld.protocol.update.HardwareVariantB\x06\n\x04\x64\x61taB\x03\xf8\x01\x01')
  ,
  dependencies=[blickfeld_dot_data_dot_frame__pb2.DESCRIPTOR,blickfeld_dot_data_dot_scanline__pb2.DESCRIPTOR,blickfeld_dot_options__pb2.DESCRIPTOR,blickfeld_dot_update_dot_hardware__pb2.DESCRIPTOR,])




_POINTCLOUD_HEADER = _descriptor.Descriptor(
  name='Header',
  full_name='blickfeld.protocol.data.PointCloud.Header',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='legacy_cube_serial_number', full_name='blickfeld.protocol.data.PointCloud.Header.legacy_cube_serial_number', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cube_serial_number', full_name='blickfeld.protocol.data.PointCloud.Header.cube_serial_number', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\320\265\030\001\252\265\030\013[A-Z2-7]{9}'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start_time_ns', full_name='blickfeld.protocol.data.PointCloud.Header.start_time_ns', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='firmware_version', full_name='blickfeld.protocol.data.PointCloud.Header.firmware_version', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\252\265\030\027v[0-9]+\\.[0-9]+\\.[0-9]+'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hardware_variant', full_name='blickfeld.protocol.data.PointCloud.Header.hardware_variant', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=5,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=360,
  serialized_end=606,
)

_POINTCLOUD = _descriptor.Descriptor(
  name='PointCloud',
  full_name='blickfeld.protocol.data.PointCloud',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='blickfeld.protocol.data.PointCloud.header', index=0,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='frame', full_name='blickfeld.protocol.data.PointCloud.frame', index=1,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='scanline', full_name='blickfeld.protocol.data.PointCloud.scanline', index=2,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_POINTCLOUD_HEADER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='blickfeld.protocol.data.PointCloud.data',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=179,
  serialized_end=614,
)

_POINTCLOUD_HEADER.fields_by_name['hardware_variant'].enum_type = blickfeld_dot_update_dot_hardware__pb2._HARDWAREVARIANT
_POINTCLOUD_HEADER.containing_type = _POINTCLOUD
_POINTCLOUD.fields_by_name['header'].message_type = _POINTCLOUD_HEADER
_POINTCLOUD.fields_by_name['frame'].message_type = blickfeld_dot_data_dot_frame__pb2._FRAME
_POINTCLOUD.fields_by_name['scanline'].message_type = blickfeld_dot_data_dot_scanline__pb2._SCANLINE
_POINTCLOUD.oneofs_by_name['data'].fields.append(
  _POINTCLOUD.fields_by_name['header'])
_POINTCLOUD.fields_by_name['header'].containing_oneof = _POINTCLOUD.oneofs_by_name['data']
_POINTCLOUD.oneofs_by_name['data'].fields.append(
  _POINTCLOUD.fields_by_name['frame'])
_POINTCLOUD.fields_by_name['frame'].containing_oneof = _POINTCLOUD.oneofs_by_name['data']
_POINTCLOUD.oneofs_by_name['data'].fields.append(
  _POINTCLOUD.fields_by_name['scanline'])
_POINTCLOUD.fields_by_name['scanline'].containing_oneof = _POINTCLOUD.oneofs_by_name['data']
DESCRIPTOR.message_types_by_name['PointCloud'] = _POINTCLOUD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PointCloud = _reflection.GeneratedProtocolMessageType('PointCloud', (_message.Message,), dict(

  Header = _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), dict(
    DESCRIPTOR = _POINTCLOUD_HEADER,
    __module__ = 'blickfeld.data.point_cloud_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.data.PointCloud.Header)
    ))
  ,
  DESCRIPTOR = _POINTCLOUD,
  __module__ = 'blickfeld.data.point_cloud_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.data.PointCloud)
  ))
_sym_db.RegisterMessage(PointCloud)
_sym_db.RegisterMessage(PointCloud.Header)


DESCRIPTOR._options = None
_POINTCLOUD_HEADER.fields_by_name['legacy_cube_serial_number']._options = None
_POINTCLOUD_HEADER.fields_by_name['cube_serial_number']._options = None
_POINTCLOUD_HEADER.fields_by_name['firmware_version']._options = None
# @@protoc_insertion_point(module_scope)



#!/usr/bin/python
#
# Copyright (c) 2020 Blickfeld GmbH.
# All rights reserved.
#
# This source code is licensed under the BSD-style license found in the
# LICENSE.md file in the root directory of this source tree.
#

from __future__ import print_function

import math
import gzip
import threading
import time
from google.protobuf.internal.decoder import _DecodeVarint32
from google.protobuf.internal.encoder import _EncodeVarint

from ..protocol import connection_pb2
from ..protocol.file import general_pb2, point_cloud_pb2
from ..protocol.data import frame_pb2
from .. import version

frame_pb2.Frame.__repr__ = frame_pb2.Frame.__str__ = lambda self: "<Blickfeld Frame %u: %u returns, %.1fx%.1f FoV, %u scanlines>" % (
        self.id, 
        self.total_number_of_returns, 
        self.scan_pattern.horizontal.fov * 180 / math.pi, 
        self.scan_pattern.vertical.fov * 180 / math.pi,
        len(self.scanlines)
    )

""" Reference frame: XYZ coordinates """
REF_FRAME_XYZ = frame_pb2.Frame()
REF_FRAME_XYZ.scan_pattern.SetInParent()
REF_FRAME_XYZ.total_number_of_points = 0
REF_FRAME_XYZ.total_number_of_returns = 0
REF_FRAME_XYZ.scanlines.add().points.add().returns.add().cartesian.append(0)

""" Reference frame: XYZ coordinates, intensity """
REF_FRAME_XYZ_I = frame_pb2.Frame()
REF_FRAME_XYZ_I.CopyFrom(REF_FRAME_XYZ)
REF_FRAME_XYZ_I.scanlines[0].points[0].returns[0].intensity = 0

""" Reference frame: XYZ coordinates, intensity, frame id, scanline id, point id, return id """
REF_FRAME_XYZ_I_ID = frame_pb2.Frame()
REF_FRAME_XYZ_I_ID.CopyFrom(REF_FRAME_XYZ_I)
REF_FRAME_XYZ_I_ID.id = 0
REF_FRAME_XYZ_I_ID.scanlines[0].id = 0
REF_FRAME_XYZ_I_ID.scanlines[0].points[0].id = 0
REF_FRAME_XYZ_I_ID.scanlines[0].points[0].returns[0].id = 0

""" Reference frame: XYZ coordinates, intensity, frame id, scanline id, point id, return id, timestamps """
REF_FRAME_XYZ_I_ID_TS = frame_pb2.Frame()
REF_FRAME_XYZ_I_ID_TS.CopyFrom(REF_FRAME_XYZ_I_ID)
REF_FRAME_XYZ_I_ID_TS.start_time_ns = 0
REF_FRAME_XYZ_I_ID_TS.scanlines[0].start_offset_ns = 0
REF_FRAME_XYZ_I_ID_TS.scanlines[0].points[0].start_offset_ns = 0

""" Reference frame: ambient_light_level, intensity, range, frame id, scanline id, point id """
REF_FRAME_DEPTH_MAP = frame_pb2.Frame()
REF_FRAME_DEPTH_MAP.scan_pattern.SetInParent()
REF_FRAME_DEPTH_MAP.total_number_of_points = 0
REF_FRAME_DEPTH_MAP.total_number_of_returns = 0
REF_FRAME_DEPTH_MAP.id = 0
REF_FRAME_DEPTH_MAP.scanlines.add().points.add().returns.add()
REF_FRAME_DEPTH_MAP.scanlines[0].id = 0
REF_FRAME_DEPTH_MAP.scanlines[0].points[0].id = 0
REF_FRAME_DEPTH_MAP.scanlines[0].points[0].ambient_light_level = 0
REF_FRAME_DEPTH_MAP.scanlines[0].points[0].returns[0].intensity = 0
REF_FRAME_DEPTH_MAP.scanlines[0].points[0].returns[0].range = 0


class point_cloud(object):
    """ Class to request a point cloud stream

    :param connection: connection to the device
    :type connection: :py:class:`blickfeld_scanner.scanner.connection`
    """

    """ Reference frame: XYZ coordinates """
    REF_FRAME_XYZ = REF_FRAME_XYZ
    """ Reference frame: XYZ coordinates, intensity """
    REF_FRAME_XYZ_I = REF_FRAME_XYZ_I
    """ Reference frame: XYZ coordinates, intensity, frame id, scanline id, point id, return id """
    REF_FRAME_XYZ_I_ID = REF_FRAME_XYZ_I_ID
    """ Reference frame: XYZ coordinates, intensity, frame id, scanline id, point id, return id, timestamps """
    REF_FRAME_XYZ_I_ID_TS = REF_FRAME_XYZ_I_ID_TS
    """ Reference frame: ambient_light_level, intensity, range, frame id, scanline id, point id """
    REF_FRAME_DEPTH_MAP = REF_FRAME_DEPTH_MAP

    def __init__(self, connection=None, from_file=None, filter=None, reference_frame=None):
        self._metadata = point_cloud_pb2.PointCloud.Metadata()
        
        if connection and from_file:
            raise AttributeError("Either provide connection or from_file.")
        if not connection and not from_file:
            raise AttributeError("Neither connection nor from_file is provided.")
           
        self._ofile = False
        self._thread = None
        self._connection = None 
        self._stream_data = point_cloud_pb2.PointCloud.Data()
        if connection:
            self._connection = connection

            req = connection_pb2.Request()
            req.subscribe.point_cloud.SetInParent()
            if filter:
                req.subscribe.point_cloud.filter.CopyFrom(filter)
            if reference_frame:
                req.subscribe.point_cloud.reference_frame.CopyFrom(reference_frame)
            
            self._metadata.header.device.CopyFrom(self._connection.send_request(req).event.point_cloud.header)
            
            self._metadata.header.client.library_version = version.__version__
            self._metadata.header.client.file_time_ns = int(time.time() * 1e9)
            self._metadata.header.client.language = general_pb2.Client.Language.PYTHON
        else:
            # Open file
            self._file_name = from_file
            self._file = gzip.open(self._file_name, 'rb')
            self._file.seek(0, 2)
            self._file_size = self._file.tell()
            self._file.seek(0, 0)
            
            # Read header
            buf_header = self._get_file_block()
            offset_data = self._file.tell()
            self._metadata.header.ParseFromString(buf_header)
            self._stream_buffered = False
            
            # Find footer offset
            while self._file_size > self._file.tell():
                offset_footer = self._file.tell()
                self._get_file_block()
                
            # Read footer
            self._file.seek(offset_footer)
            if self._read_stream():
                self._metadata.footer.CopyFrom(self._stream_data.footer)
                self._stream_buffered = False
            self._file.seek(offset_data)

    def __del__(self):
        self.close()
        
    def __str__(self):
        return "<Blickfeld Point Cloud Stream: %u frames, %u returns, %s>" % (
            self._metadata.footer.stats.counter.frames,
            self._metadata.footer.stats.counter.returns,
            ("device '%s'" % (self._metadata.header.device.cube_serial_number)) if self._connection else ("file '%s'" % (self._file_name))
        )
        
    def _run(self):
        self._prev_scan_pattern_str = b""
    
        while not self._thread_stop:
            with self._thread_sync:
                self._thread_sync.wait(0.1)
                
                if self._thread_stop:
                    break
                
                # Check if main thread is still active.
                main_thread = None
                for i in threading.enumerate():
                    if i.name == "MainThread":
                        main_thread = i
                if not main_thread.is_alive():
                    break

                if self._stream_data.HasField("frame"):
                    # Add events for changed scan pattern to footer
                    scan_pattern_str = self._stream_data.frame.scan_pattern.SerializeToString()
                    if self._prev_scan_pattern_str != scan_pattern_str:
                        event = self._metadata.footer.events.add()
                        event.from_frame_id = self._stream_data.frame.id
                        event.scan_pattern.CopyFrom(self._stream_data.frame.scan_pattern)
                        
                        self._prev_scan_pattern_str = scan_pattern_str
                
                    # Update stats in footer
                    counter = self._metadata.footer.stats.counter
                    counter.frames = counter.frames + 1
                    counter.points = counter.points + self._stream_data.frame.total_number_of_points
                    counter.returns = counter.returns + self._stream_data.frame.total_number_of_returns
                
                    _EncodeVarint(self._ofile.write, self._stream_data.ByteSize())
                    self._ofile.write(self._stream_data.SerializeToString())
                    # Clear data
                    self._stream_data.ClearField("frame")

    def close(self):
        """ Close point cloud stream and connection
        """
        self.stop_recording()
        
        if self._connection:
            self._connection.close()
        else:
            self._file.close()
            
    def _get_file_block(self):
        buf = self._file.read(4)
        len, offset = _DecodeVarint32(buf, 0)
        buf = buf[offset:] + self._file.read(len - 4 + offset)
        return buf
    
    def _read_stream(self):
        if self._file_size == self._file.tell():
            self._stream_buffered = False
            return False
        
        self._stream_data.Clear()
        self._stream_data.ParseFromString(self._get_file_block())
        self._stream_buffered = True
        return self._stream_buffered

    def get_metadata(self):
        """ Get metadata of point cloud stream

        :return: Point cloud metadata, see: :any:`protobuf_protocol` PointCloud.Metadata
        """
        return self._metadata

    def recv_frame(self):
        """ Receive point cloud frame

        :return: point cloud frame with all the data in it, see: :any:`protobuf_protocol` Frame
        """
        if self._connection:
            frame = self._connection.recv().event.point_cloud.frame
        else:
            if self.end_of_stream():
                raise Exception("Reached end of stream. Use end_of_stream() before calling recv_frame().")
            self._stream_buffered = False
                
            frame = self._stream_data.frame
        
        # Record frame to file
        if self._ofile:
            with self._thread_sync:
                self._stream_data.frame.CopyFrom(frame)
                self._thread_sync.notify()
        
        return frame
        
    def end_of_stream(self):
        """ Check whether stream has ended. This function is required for point cloud streams of files

        :return: Bool if end of stream is reached
        """
        if not self._stream_buffered:
            self._read_stream()
        return not self._stream_buffered or self._stream_data.HasField('footer')

    def record_to_file(self, file_name):
        """ Record point cloud stream to file

        :param file_name: Path to the file where it should be dumped
        """
        self._ofile = gzip.open(file_name, 'wb')
        _EncodeVarint(self._ofile.write, self._metadata.header.ByteSize())
        self._ofile.write(self._metadata.header.SerializeToString())

        self._thread = threading.Thread(target=self._run)
        self._thread_sync = threading.Condition()
        self._thread_stop = False
        self._thread.start()

    def stop_recording(self):
        """ Stop recording
        """
        if self._thread and self._thread.isAlive():
            self._thread_stop = True
            with self._thread_sync:
                self._thread_sync.notify()
            self._thread.join()

        if self._ofile:
            self._stream_data.footer.CopyFrom(self._metadata.footer)
            _EncodeVarint(self._ofile.write, self._stream_data.ByteSize())
            self._ofile.write(self._stream_data.SerializeToString())

            self._ofile.close()
            self._ofile = None



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/common.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import options_pb2 as blickfeld_dot_options__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/common.proto',
  package='blickfeld.protocol',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=_b('\n\x16\x62lickfeld/common.proto\x12\x12\x62lickfeld.protocol\x1a\x17\x62lickfeld/options.proto\".\n\nValueRange\x12\x0f\n\x07minimum\x18\x01 \x01(\x02\x12\x0f\n\x07maximum\x18\x02 \x01(\x02\"B\n\x12OptionalValueRange\x12\x15\n\x07minimum\x18\x01 \x01(\x02\x42\x04\xb0\xb5\x18\x01\x12\x15\n\x07maximum\x18\x02 \x01(\x02\x42\x04\xb0\xb5\x18\x01')
  ,
  dependencies=[blickfeld_dot_options__pb2.DESCRIPTOR,])




_VALUERANGE = _descriptor.Descriptor(
  name='ValueRange',
  full_name='blickfeld.protocol.ValueRange',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='minimum', full_name='blickfeld.protocol.ValueRange.minimum', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maximum', full_name='blickfeld.protocol.ValueRange.maximum', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=71,
  serialized_end=117,
)


_OPTIONALVALUERANGE = _descriptor.Descriptor(
  name='OptionalValueRange',
  full_name='blickfeld.protocol.OptionalValueRange',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='minimum', full_name='blickfeld.protocol.OptionalValueRange.minimum', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maximum', full_name='blickfeld.protocol.OptionalValueRange.maximum', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=119,
  serialized_end=185,
)

DESCRIPTOR.message_types_by_name['ValueRange'] = _VALUERANGE
DESCRIPTOR.message_types_by_name['OptionalValueRange'] = _OPTIONALVALUERANGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ValueRange = _reflection.GeneratedProtocolMessageType('ValueRange', (_message.Message,), dict(
  DESCRIPTOR = _VALUERANGE,
  __module__ = 'blickfeld.common_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.ValueRange)
  ))
_sym_db.RegisterMessage(ValueRange)

OptionalValueRange = _reflection.GeneratedProtocolMessageType('OptionalValueRange', (_message.Message,), dict(
  DESCRIPTOR = _OPTIONALVALUERANGE,
  __module__ = 'blickfeld.common_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.OptionalValueRange)
  ))
_sym_db.RegisterMessage(OptionalValueRange)


_OPTIONALVALUERANGE.fields_by_name['minimum']._options = None
_OPTIONALVALUERANGE.fields_by_name['maximum']._options = None
# @@protoc_insertion_point(module_scope)
